// Init FSM for DDR SDRAM
//


module ddr_init_fsm (
fsm_init_mrs, fsm_clk_in, fsm_init_start, fsm_init_in,
fsm_init_out, fsm_init_done
);


	// Format of output is: {cs,ras,cas,we,ba[1:0],a[12:0]}
parameter [18:0]
		NOP_CMD_I		= 19'b0111xxxxxxxxxxxxxxx,
		PREA_CMD_I		= 19'b0010xxxxx1xxxxxxxxx,
		EMRS_CMD_I		= 19'b0000010000000000000,
		AR_CMD_I		= 19'b0001xxxxxxxxxxxxxxx;

// Timing parameter for init commands
parameter [7:0]
		T_PRE1		= 8'h04,	// Time at the beginning of the command 
		T_EMRS		= 8'h06,
		T_MRS		= 8'h08,
		T_PRE2		= 8'h0A,
		T_AR1		= 8'h0C,
		T_AR2		= 8'h13,
		T_RDY		= 8'h20;

// Index into the state register
parameter [2:0]
		NOP_I 	= 0,
		PRE_I 	= 1,
		EMRS_I 	= 2,
		MRS_I 	= 3,
		AR_I 	= 4,
		RDY_I	= 5;

input [14:0] fsm_init_mrs;
input fsm_clk_in;	
input fsm_init_start;
input [7:0] fsm_init_in;	// Clock counts - assume freq = 100MHz
output [18:0] fsm_init_out; // Commands to DDR
output fsm_init_done;

reg [5:0] fsm_init_state;
reg [5:0] fsm_init_next_state;
reg [18:0] fsm_init_out;
reg fsm_init_done;

always @ (fsm_init_state or fsm_init_in)	// Block for FSM states
  begin
    fsm_init_next_state = 6'b000000;	// Must be zero at the beginning of the cycle
    fsm_init_out = NOP_CMD_I;
    fsm_init_done = 1'b0;
    case (1'b1)
      fsm_init_state[NOP_I]: 
        begin
          fsm_init_out = NOP_CMD_I;
          case (fsm_init_in)
            T_PRE1, T_PRE2: fsm_init_next_state[PRE_I] = 1'b1;
            T_EMRS: fsm_init_next_state[EMRS_I] = 1'b1;
            T_MRS: fsm_init_next_state[MRS_I] = 1'b1;
            T_AR1, T_AR2: fsm_init_next_state[AR_I] = 1'b1;
            T_RDY: fsm_init_next_state[RDY_I] = 1'b1;
            default: fsm_init_next_state[NOP_I] = 1'b1;
          endcase
        end
      fsm_init_state[PRE_I]:
        begin
          fsm_init_out = PREA_CMD_I;
          fsm_init_next_state[NOP_I] = 1'b1;
        end
      fsm_init_state[EMRS_I]:
        begin
          fsm_init_out = EMRS_CMD_I;
          fsm_init_next_state[NOP_I] = 1'b1;
        end
      fsm_init_state[MRS_I]:
        begin
          fsm_init_out[18:0] = {4'b0000,fsm_init_mrs[14:0]};
          fsm_init_next_state[NOP_I] = 1'b1;
        end
      fsm_init_state[AR_I]:
        begin
          fsm_init_out = AR_CMD_I;
          fsm_init_next_state[NOP_I] = 1'b1;
        end
      fsm_init_state[RDY_I]:
        begin
          fsm_init_done = 1'b1;
          fsm_init_next_state[RDY_I] = 1'b1;
        end
    endcase
  end

always @ (negedge fsm_clk_in)
  begin
    if (!fsm_init_start)
      begin
        fsm_init_state <= 6'b000000;
        fsm_init_state[NOP_I] <= 1'b1;
      end
    else
      fsm_init_state <= fsm_init_next_state;
  end

endmodule
